# include <math.h>

float ComponentToFraction(UWORD comp)
{
	return ((float) comp) / 15.0f;
}

UWORD FractionToComponent(float frac)
{
	return (UWORD) round(frac * 15.0f);
}

void Separate(UWORD col, UWORD *r, UWORD *g, UWORD *b)
{
	*r = (col & 0xf00) >> 8;
	*g = (col & 0x0f0) >> 4;
	*b = col & 0x00f;
}

void SeparateFractions(UWORD col, float *r, float *g, float *b)
{
	*r = ComponentToFraction((col & 0xf00) >> 8);
	*g = ComponentToFraction((col & 0x0f0) >> 4);
	*b = ComponentToFraction(col & 0x00f);
}

UWORD Combine(UWORD r, UWORD g, UWORD b)
{
	return (r << 8) + (g << 4) + b;
}

UWORD CombineFractions(float r, float g, float b)
{
	return (FractionToComponent(r) << 8) + (FractionToComponent(g) << 4) + FractionToComponent(b);
}

void Represent(UWORD col, char * dest)
{
	UWORD r, g, b;
	Separate(col, &r, &g, &b);
	sprintf(dest, "RGB(%.2d, %.2d, %.2d)", r, g, b);
}

UWORD Blend(UWORD col0, UWORD col1, float bias)
{
	float r0, g0, b0;
	float r1, g1, b1;
	SeparateFractions(col0, &r0, &g0, &b0);
	SeparateFractions(col1, &r1, &g1, &b1);
	
	float bias1 = 1.0f - bias;
	float nr = (bias1 * r0) + (bias * r1);
	float ng = (bias1 * g0) + (bias * g1);
	float nb = (bias1 * b0) + (bias * b1);
	
	return CombineFractions(nr, ng, nb);
}

BOOL SampleGradient(UWORD col0, UWORD col1, UWORD nrSamples, UWORD * samples)
{
	if(nrSamples < 1) return FALSE;
	
	if(nrSamples < 2)
	{
		samples[0] = Blend(col0, col1, 0.5f);
		return TRUE;
	}
	
	if(nrSamples < 3)
	{
		samples[0] = col0;
		samples[1] = col1;
		return TRUE;
	}
	
	// common case: 3 or more samples
	samples[0] = col0;
	samples[nrSamples - 1] = col1;
	
	float deltaBias = 1.0f / ((float) (nrSamples - 1));
	float currBias;
	for(int iblend = 1; iblend <= (nrSamples - 2); iblend++)
	{
		currBias = deltaBias * ((float) iblend);
		samples[iblend] = Blend(col0, col1, currBias);
	}
	
	return TRUE;
}

void PrintGradient(UWORD * colors, UWORD nrColors)
{
	char strCol[] = "RGB(15, 15, 15)";
	for(UWORD iColor = 0; iColor < nrColors; iColor++)
	{
		Represent(colors[iColor], strCol);
		printf("-- %.3d: %s\n", iColor, strCol);
	}
}

void MirrorGradient(UWORD * src, UWORD * dst, UWORD nrColors)
{
	UWORD nrColors1 = nrColors - 1;
	for(UWORD iSrc = 0; iSrc < nrColors; iSrc++)
	{
		dst[nrColors1 - iSrc] = src[iSrc];
	}
}

// dst has src appended to it -- so it should be big enough
void AppendGradient(UWORD * dst, UWORD * src, UWORD nrDst, UWORD nrSrc, BOOL skip1)
{
	UWORD srcStart = 1;
	if(!skip1) srcStart = 0;
	UWORD iDst = nrDst;
	for(UWORD iSrc = srcStart; iSrc < nrSrc; iSrc++)
	{
		dst[iDst] = src[iSrc];
		iDst++;
	}
}

void TestConversion()
{
	float frac;
	UWORD ncomp;
	for(UWORD comp = 0; comp < 16; comp++)
	{
		frac = ComponentToFraction(comp);
		ncomp = FractionToComponent(frac);
		printf("%.2d -- %f -- %2.d\n", comp, frac, ncomp);
	}
}

void TestRepresent()
{
	UWORD col = 0xfa5;
	// char str[] = "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ";
	char str[] = "RGB(15, 15, 15)";
	Represent(col, str);
	printf(str);
	printf("\n");
}

void TestGradient()
{
	UWORD nrColors = 16;
	UWORD colors[nrColors];
	UWORD col0 = 0x000;
	UWORD col1 = 0xfff;
	
	if(!SampleGradient(col0, col1, nrColors, colors))
	{
		printf("?? ERROR in SampleGradient()");
		return;
	}
	
	PrintGradient(colors, nrColors);
}